package org.apache.lucene.analysis;

import java.io.IOException;

import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
import org.apache.lucene.analysis.tokenattributes.KeywordAttribute;
import org.apache.lucene.analysis.tokenattributes.TypeAttribute;

public class SuffixKeywordFilter  extends TokenFilter {
    private final CharTermAttribute termAtt = addAttribute(CharTermAttribute.class);
    private final TypeAttribute typeAtt = addAttribute(TypeAttribute.class);
//    private final PositionIncrementAttribute posIncAtt = addAttribute(PositionIncrementAttribute.class);
    private final KeywordAttribute keywordAtt = addAttribute(KeywordAttribute.class);

    private final Character suffix;

    public SuffixKeywordFilter(final TokenStream input, final Character suffixToAdd) {
        super(input);
        this.suffix = suffixToAdd;
    }

    @Override
    public final boolean incrementToken() throws IOException {
        if (!input.incrementToken()) { // reached EOS -- return null
            return false;
        }

        // If the filter is disabled, or this is not a keyword, skip and allow the stem / bigram to be returned
        if (!keywordAtt.isKeyword() || suffix == null || CommonGramsFilter.GRAM_TYPE.equals(typeAtt.type())) {
            return true;
        }

        termAtt.append(suffix);

        return true;
    }
}